<?php
/*********************************************************************************
 * The contents of this file are subject to the TimeTrex Public License Version
 * 1.1.0 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.TimeTrex.com/TPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by TimeTrex" logo and
 *    (ii) the TimeTrex copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: TimeTrex Open Source
 * The Initial Developer of the Original Code is TimeTrex Payroll Services
 * Portions created by TimeTrex are Copyright (C) 2004-2007 TimeTrex Payroll Services;
 * All Rights Reserved.
 *
 ********************************************************************************/
/*
 * $Revision: 417 $
 * $Id: UserList.php 417 2006-12-06 22:58:53Z ipso $
 * $Date: 2006-12-06 14:58:53 -0800 (Wed, 06 Dec 2006) $
 */
require_once('../../includes/global.inc.php');
require_once(Environment::getBasePath() .'includes/Interface.inc.php');

//Debug::setVerbosity( 11 );

if ( !$permission->Check('user','enabled')
		OR !( $permission->Check('user','view') OR $permission->Check('user','view_child')  ) ) {
	$permission->Redirect( FALSE ); //Redirect
}

$title = 'Employee List';
$smarty->assign('title', $title);
BreadCrumb::setCrumb($title);

/*
 * Get FORM variables
 */
extract	(FormVariables::GetVariables(
										array	(
												'action',
												'form',
												'page',
												'filter_data',
												'sort_column',
												'sort_order',
												'saved_search_id',
												'ids'
												) ) );

$columns = array(
											'-1010-employee_number' => 'Employee #',
											'-1020-status' => 'Status',
											'-1030-user_name' => 'User Name',
											'-1040-phone_id' => 'Phone ID',
											'-1050-ibutton_id' => 'iButton',

											'-1060-first_name' => 'First Name',
											'-1070-middle_name' => 'Middle Name',
											'-1080-last_name' => 'Last Name',

											'-1090-title' => 'Title',

											'-1099-user_group' => 'Group',
											'-1100-default_branch' => 'Branch',
											'-1110-default_department' => 'Department',

											'-1120-sex' => 'Sex',

											'-1130-address1' => 'Address 1',
											'-1140-address2' => 'Address 2',

											'-1150-city' => 'City',
											'-1160-province' => 'Province/State',
											'-1170-country' => 'Country',
											'-1180-postal_code' => 'Postal Code',
											'-1190-work_phone' => 'Work Phone',
											'-1200-home_phone' => 'Home Phone',
											'-1210-mobile_phone' => 'Mobile Phone',
											'-1220-fax_phone' => 'Fax Phone',
											'-1230-home_email' => 'Home Email',
											'-1240-work_email' => 'Work Email',
											'-1250-birth_date' => 'Birth Date',
											'-1260-hire_date' => 'Hire Date',
											'-1270-termination_date' => 'Termination Date',
											'-1280-sin' => 'SIN/SSN',
											);

if ( $saved_search_id == '' AND !isset($filter_data['columns']) ) {
	//Default columns.
	$filter_data['columns'] = array(
								'-1060-first_name',
								'-1080-last_name',
								'-1200-home_phone'
								);
	if ( $sort_column == '' ) {
		$sort_column = $filter_data['sort_column'] = 'last_name';
		$sort_order = $filter_data['sort_order'] = 'asc';
	}
}

$ugdlf = new UserGenericDataListFactory();
$ugdf = new UserGenericDataFactory();

//Get Permission Hierarchy Children first, as this can be used for viewing, or editing.
$hlf = new HierarchyListFactory();
$permission_children_ids = $hlf->getHierarchyChildrenByCompanyIdAndUserIdAndObjectTypeID( $current_company->getId(), $current_user->getId() );
Debug::Arr($permission_children_ids,'Permission Children Ids:', __FILE__, __LINE__, __METHOD__,10);

//Handle different actions for different forms.
if ( isset($form) AND $form != '' ) {
	$action = strtolower($form.'_'.$action);
} else {
	$action = strtolower($action);
}
Debug::Text('Action: '. $action, __FILE__, __LINE__, __METHOD__,10);

switch ($action) {
	case 'add':

		Redirect::Page( URLBuilder::getURL( array('saved_search_id' => $saved_search_id ), 'EditUser.php') );

		break;
	case 'delete':
	case 'undelete':
		//Debug::setVerbosity( 11 );
		if ( strtolower($action) == 'delete' ) {
			$delete = TRUE;
		} else {
			$delete = FALSE;
		}

		if ( DEMO_MODE == FALSE
			AND ( $permission->Check('user','delete') OR $permission->Check('user','delete_own') OR $permission->Check('user','delete_child')  ) ) {
			$ulf = new UserListFactory();
			$ulf->StartTransaction();

			foreach ($ids as $id) {
				if ( $id != $current_user->getId() ) {
					$ulf->getByIdAndCompanyId($id, $current_company->getID() );
					foreach ($ulf as $user) {
						$is_owner = $permission->isOwner( $user->getCreatedBy(), $user->getID() );
						$is_child = $permission->isChild( $user->getId(), $permission_children_ids );

						if ( $permission->Check('user','delete')
								OR ( $permission->Check('user','delete_child') AND $is_child === TRUE )
								OR ( $permission->Check('user','delete_own') AND $is_owner === TRUE ) ) {
							$user->setDeleted($delete);
							$user->Save();
						}
					}
				}
			}
			$ulf->CommitTransaction();
		}

		Redirect::Page( URLBuilder::getURL( array('saved_search_id' => $saved_search_id ), 'UserList.php') );

		break;
	case 'search_form_delete':
	case 'search_form_update':
	case 'search_form_save':
	case 'search_form_clear':
	case 'search_form_search':
		Debug::Text('Action: '. $action, __FILE__, __LINE__, __METHOD__,10);

		$saved_search_id = UserGenericDataFactory::searchFormDataHandler( $action, $filter_data, URLBuilder::getURL(NULL, 'UserList.php') );
	default:
		extract( UserGenericDataFactory::getSearchFormData( $saved_search_id, $sort_column ) );
		Debug::Text('Sort Column: '. $sort_column, __FILE__, __LINE__, __METHOD__,10);
		Debug::Text('Saved Search ID: '. $saved_search_id, __FILE__, __LINE__, __METHOD__,10);

		$sort_array = NULL;
		if ( $sort_column != '' ) {
			$sort_array = array(Misc::trimSortPrefix($sort_column) => $sort_order);
		}

		URLBuilder::setURL($_SERVER['SCRIPT_NAME'],	array(
															'sort_column' => Misc::trimSortPrefix($sort_column),
															'sort_order' => $sort_order,
															'saved_search_id' => $saved_search_id,
															'page' => $page
														) );

		$ulf = new UserListFactory();
		$clf = new CompanyListFactory();

		if ( $permission->Check('company','view') ) {
			$ulf->getAll( $current_user_prefs->getItemsPerPage(), $page, NULL, $sort_array );
		} else {
			if ( $permission->Check('user','view') == FALSE ) {
				if ( $permission->Check('user','view_child') ) {
					$filter_data['permission_children_ids'] = $permission_children_ids;
				}
				if ( $permission->Check('user','view_own') ) {
					$filter_data['permission_children_ids'][] = $current_user->getId();
				}
			}

			Debug::Text('Users in company only!', __FILE__, __LINE__, __METHOD__,10);
			$ulf->getSearchByCompanyIdAndArrayCriteria( $current_company->getId(), $filter_data, $current_user_prefs->getItemsPerPage(), $page, NULL, $sort_array );
		}

		$pager = new Pager($ulf);

		//Get title list,
		$utlf = new UserTitleListFactory();
		$utlf->getByCompanyId( $current_company->getId() );
		$title_options = $utlf->getArrayByListFactory( $utlf, FALSE, TRUE );

		$blf = new BranchListFactory();
		$blf->getByCompanyId( $current_company->getId() );
		$branch_options = $blf->getArrayByListFactory( $blf, FALSE, TRUE );

		$dlf = new DepartmentListFactory();
		$dlf->getByCompanyId( $current_company->getId() );
		$department_options = $dlf->getArrayByListFactory( $dlf, FALSE, TRUE );

		$uglf = new UserGroupListFactory();
		$group_options = $uglf->getArrayByNodes( FastTree::FormatArray( $uglf->getByCompanyIdArray( $current_company->getId() ), 'TEXT', TRUE) );


		foreach ($ulf as $u_obj) {
			$company_name = $clf->getById( $u_obj->getCompany() )->getCurrent()->getName();

			$users[] = array(
									'id' => $u_obj->getId(),
									'employee_number' => $u_obj->getEmployeeNumber(),
									'status_id' => $u_obj->getStatus(),
									'status' => Option::getByKey( $u_obj->getStatus(), $u_obj->getOptions('status') ),
									'user_name' => $u_obj->getUserName(),
									'phone_id' => $u_obj->getPhoneID(),
									'ibutton_id' => $u_obj->getIButtonID(),

									'full_name' => $u_obj->getFullName(TRUE),
									'first_name' => $u_obj->getFirstName(),
									'middle_name' => $u_obj->getMiddleName(),
									'last_name' => $u_obj->getLastName(),

									'title' => Option::getByKey($u_obj->getTitle(), $title_options ),
									'user_group' => Option::getByKey($u_obj->getGroup(), $group_options ),

									'default_branch' => Option::getByKey($u_obj->getDefaultBranch(), $branch_options ),
									'default_department' => Option::getByKey($u_obj->getDefaultDepartment(), $department_options ),

									'sex_id' => $u_obj->getSex(),
									'sex' => Option::getByKey($u_obj->getSex(), $u_obj->getOptions('sex') ),

									'address1' => $u_obj->getAddress1(),
									'address2' => $u_obj->getAddress2(),
									'city' => $u_obj->getCity(),
									'province' => $u_obj->getProvince(),
									'country' => $u_obj->getCountry(),
									'postal_code' => $u_obj->getPostalCode(),
									'work_phone' => $u_obj->getWorkPhone(),
									'home_phone' => $u_obj->getHomePhone(),
									'mobile_phone' => $u_obj->getMobilePhone(),
									'fax_phone' => $u_obj->getFaxPhone(),
									'home_email' => $u_obj->getHomeEmail(),
									'work_email' => $u_obj->getWorkEmail(),
									'birth_date' => TTDate::getDate('DATE', $u_obj->getBirthDate() ),
									'sin' => $u_obj->getSIN(),
									'hire_date' => TTDate::getDate('DATE', $u_obj->getHireDate() ),
									'termination_date' => TTDate::getDate('DATE', $u_obj->getTerminationDate() ),

									'is_owner' => $permission->isOwner( $u_obj->getCreatedBy(), $u_obj->getId() ),
									'is_child' => $permission->isChild( $u_obj->getId(), $permission_children_ids ),
									'deleted' => $u_obj->getDeleted(),
							);
		}
		//var_dump($users);

		$all_array_option = array('-1' => '-- Any --');

		//Select box options;
		$filter_data['branch_options'] = Misc::prependArray( $all_array_option, $branch_options );
		$filter_data['department_options'] = Misc::prependArray( $all_array_option, $department_options );
		$filter_data['title_options'] = Misc::prependArray( $all_array_option, $title_options );
		$filter_data['group_options'] = Misc::prependArray( $all_array_option, $group_options );
		$filter_data['sex_options'] = Misc::prependArray( $all_array_option, $ulf->getOptions('sex') );
		$filter_data['status_options'] = Misc::prependArray( $all_array_option, $ulf->getOptions('status') );

		$cf = new CompanyFactory();
		$filter_data['country_options'] = Misc::prependArray( $all_array_option, $cf->getOptions('country') );
		if ( isset($filter_data['country']) ) {
			$filter_data['province_options'] = Misc::prependArray( $all_array_option, $cf->getOptions('province', $filter_data['country'] ) );
		} else {
			$filter_data['province_options'] = $all_array_option;
		}

		$ppslf = new PayPeriodScheduleListFactory();
		$filter_data['pay_period_schedule_options'] = Misc::prependArray( $all_array_option, $ppslf->getByCompanyIDArray( $current_company->getId() ) );

		$pglf = new PolicyGroupListFactory();
		$filter_data['policy_group_options'] = Misc::prependArray( $all_array_option, $pglf->getByCompanyIDArray( $current_company->getId() ) );

		$filter_data['saved_search_options'] = $ugdlf->getArrayByListFactory( $ugdlf->getByUserIdAndScript( $current_user->getId(), $_SERVER['SCRIPT_NAME']), FALSE );

		//Get column list
		$filter_data['src_column_options'] = Misc::arrayDiffByKey( (array)$filter_data['columns'], $columns );
		$filter_data['selected_column_options'] = Misc::arrayIntersectByKey( (array)$filter_data['columns'], $columns );

		$filter_data['sort_options'] = Misc::trimSortPrefix($columns);
		$filter_data['sort_direction_options'] = Misc::getSortDirectionArray(TRUE);

		foreach( $filter_data['columns'] as $column_key ) {
			$filter_columns[Misc::trimSortPrefix($column_key)] = $columns[$column_key];
		}
		unset($column_key);

		$smarty->assign_by_ref('users', $users);
		$smarty->assign_by_ref('filter_data', $filter_data);
		$smarty->assign_by_ref('columns', $filter_columns );
		$smarty->assign('total_columns', count($filter_columns)+3 );

		$smarty->assign_by_ref('sort_column', $sort_column );
		$smarty->assign_by_ref('sort_order', $sort_order );
		$smarty->assign_by_ref('saved_search_id', $saved_search_id );

		$smarty->assign_by_ref('paging_data', $pager->getPageVariables() );

		break;
}
$smarty->display('users/UserList.tpl');
?>